from ll.xist import xsc, sims

"""
namespace for Relax NG files
"""

class relax(xsc.Element):
	""" 
	abstract basis class 
	"""
	register = False
	
	class Attrs(xsc.Element.Attrs):
		class ns(xsc.TextAttr): pass
		# ns is inherited from the nearest ancestor, where a ns-attribute is defined, else it's ""
		class datatypeLibrary(xsc.URLAttr): pass
		
class anyName(relax):
	"""
	matches any name from any namespace
	Restr.: name classes of attributes can't overlap
	"""


class attribute(relax):
	"""
	attribute
	Restr.: after simplification: contain only text-node-relevent nodes
	        can't be duplicated
	        infinite name classes must be enclosed in oneOrMore/zeroOrMore
	        name and name class can't be both specified
	"""

	class Attrs(relax.Attrs):
		class name(xsc.TextAttr): pass


class choice(relax):
	"""
	nameclass: a name matches choice if, and only if, it matches at least one of the subname classes
	pattern: it matches a node if, and only if, at least one of its subpatterns matches the node
	"""


class data(relax):
	"""
	specifying data of a certain kind
	Restr.: cant' be used in mixed content models
	"""

	class Attrs(relax.Attrs):
		class type(xsc.TextAttr): pass


class define(relax):
	"""
	defines a part of a grammar pattern (also a pattern), recursion possible only inside an element
	Restr.: is not possible to define and make reference to nonpatterns such as class names/datatype parameters
	"""
	
	class Attrs(relax.Attrs):
		class name(xsc.TextAttr): required = True
		class combine(xsc.TextAttr): values = [u"interleave", u"choice"]
		# values: interleave or choice
		
		
class div(relax):
	"""
	allows logical divisions , no effect on validation, annotations can be made here
	"""
	
	class Attrs(relax.Attrs):
		class name(xsc.TextAttr): pass


class element_(relax):
	"""
	element
	"""
	xmlname="element"
	
	class Attrs(relax.Attrs):
		class name(xsc.TextAttr): pass
		# alternative: namespace name


class empty(relax):
	"""
	empty element 
	"""
	model = sims.Empty()


class except_(relax):
	"""
	name-class: the except name class can remove a name class from another (this class has no attributes)
	pattern: the except pattern is used to remove a set of values from a data pattern
	"""
	xmlname = "except"


class externalRef(relax):
	"""
	reference to an extern pattern stored in a file
	"""
	model = sims.Empty()

	class Attrs(relax.Attrs):
		class href(xsc.URLAttr): required = True


class grammar(relax):
	"""
	A grammar element has a single start child element, and zero or more define child elements. The start and define elements contain patterns. These patterns can contain ref elements that refer to patterns defined by any of the define elements in that grammar element. A grammar pattern is matched by matching the pattern contained in the start element.
	"""


class group(relax):
	"""
	is implied, can be also explicitly specified: the patterns have to appear in the specified order (except for the attributes, the are allowed to appear in any order in the start tag)
	"""


class include(relax):
	"""
	to include an extern grammar pattern
	can contain define parts to overwrite that part (same name) in the extern pattern
	a possible start element inside include overwrites the start element of the extern pattern
	"""

	class Attrs(relax.Attrs):
		class href(xsc.URLAttr): required = True


class interleave(relax):
	"""
	childelements can appear in any order, if one is a group, the order must be kept, other direct childs can mix between
	Restr.: interleave can't be unsed within a list, elements within can't have overlapping name classes, there can be at most one text pattern in each set of patterns combined by interleave
	"""


class list(relax):
	"""
	to match whitespace seperated values
	Restr.: interleave can't be used within list, no list in list, content: only about data: no elment, attribut, text
	"""


class mixed(relax):
	"""
	<mixed> p </mixed> is short for <interleave> <text/> p </interleave>
	Restr.: can't be used within a list, elements within can't have overlapping name classes, there must no other text pattern in each set of patterns combined by mixed
	"""
		

class name(relax):
	"""
	defines a class with a single name
	"""
	model = sims.NoElements()


class notAllowed(relax):
	"""
	Used to make extension points in patterns (in one pattern it isn't allowed to extend here, if we combine subpatterns, it is maybe
	"""
	model = sims.Empty()


class nsName(relax):
	"""
	allows any name in a specific namespace
	Restr.: within the scope of an element, the attributes can't overlap, same when elements are combined by inerleave
	"""
	model = sims.Elements(except_)

	class Attrs(relax.Attrs):
		class ns(xsc.URLAttr): pass


class oneOrMore(relax):
	"""
	there can be one or more recurrence of the enclosed pattern
	Restr.: can't contain attribute definitions
	"""
	


class optional(relax):
	"""
	the enclosed tags can be left out
	"""


class param(relax):
	"""
	specifies parameters passed to the datatype library to determine whether a value is valid per a datatyp
	"""
	model = sims.NoElements()

	class Attrs(relax.Attrs):
		class name(xsc.TextAttr): pass


class parentRef(relax):
	"""
	escapes out of the current grammar and references a definition from the parent of the current grammar
	"""
	model = sims.Empty()

	class Attrs(relax.Attrs):
		class name(xsc.TextAttr): required = True


class ref(relax):
	"""
	A ref pattern refers to a definition from nearest grammar ancestor
	"""
	model = sims.Empty()

	class Attrs(relax.Attrs):
		class name(xsc.TextAttr): required = True


class start(relax):
	"""
	required start tag inside a grammar tag
	"""

	class Attrs(relax.Attrs):
		class combine(xsc.TextAttr): pass
			# values: combine oder interleave


class text(relax):
	"""
	matches arbitrary text (one or more text nodes), including empty text
	Restr.: No more than one text pattern in an interleave pattern
	"""
	model = sims.Empty()


class value(relax):
	"""
	By default, the value pattern will consider the string in the pattern to match the string in the document if the two strings are the same after the whitespace in both strings is normalized. Whitespace normalization strips leading and trailing whitespace characters, and collapses sequences of one or more whitespace characters to a single space character. This corresponds to the behaviour of an XML parser for an attribute that is declared as other than CDATA
	Restr.: meant for data-oriented applicated, can't be used in mixed-content
	"""
	model = sims.NoElements()

	class Attrs(relax.Attrs):
		class type(xsc.TextAttr): pass
#@TODO: evtl. kein TextAttr		


class zeroOrMore(relax):
	"""
	there can be zero or more recurrence of the enclosed pattern
	Restr.: can't contain attribute definitions
	"""

# modeltype definitions
anyName.model = sims.Elements(except_)
attribute.model = sims.ElementsOrText(choice, data, empty, externalRef, grammar, group, interleave, list, mixed, name, notAllowed, nsName, oneOrMore, optional, parentRef, ref, text, value, zeroOrMore)
choice.model = sims.Elements(anyName, attribute, choice, data, element_, empty, externalRef, grammar, group, interleave, list, mixed, name, notAllowed, nsName, oneOrMore, optional, parentRef, ref, text, value, zeroOrMore)
data.model = sims.Elements(except_, param)
define.model = sims.Elements(attribute, choice, data, element_, empty, externalRef, grammar, group, interleave, list, mixed, notAllowed, oneOrMore, optional, parentRef, ref, text, value, zeroOrMore)
div.model = sims.Elements(define, div, include, start)
element_.model = \
except_.model = sims.Elements(anyName, attribute, choice, data, element_, empty, externalRef, grammar, group, interleave, list, mixed, name, notAllowed, nsName, oneOrMore, optional, parentRef, ref, text, value, zeroOrMore)
grammar.model = sims.Elements(start, define, div, include)
group.model = sims.Elements(attribute, choice, data, element_, externalRef, grammar, group, interleave, list, mixed, notAllowed, oneOrMore, optional, parentRef, ref, text, value, zeroOrMore)
include.model = sims.Elements(start, define, div)
interleave.model = sims.Elements(attribute, choice, data, element_, externalRef, grammar, group, interleave, list, mixed, notAllowed, oneOrMore, optional, parentRef, ref, text, value, zeroOrMore)
list.model = sims.Elements(choice, data, empty, externalRef, grammar, group, notAllowed, oneOrMore, optional, parentRef, ref, value, zeroOrMore)
mixed.model = sims.Elements(attribute, choice, data, element_, empty, externalRef, grammar, group, interleave, list, mixed, notAllowed, oneOrMore, optional, parentRef, ref, text, value, zeroOrMore)
oneOrMore.model = sims.Elements(choice, data, element_, empty, externalRef, grammar, group, interleave, list, mixed, notAllowed, oneOrMore, optional, parentRef, ref, text, value, zeroOrMore)
optional.model = sims.Elements(attribute, choice, data, element_, empty, externalRef, grammar, group, interleave, list, mixed, notAllowed, oneOrMore, optional, parentRef, ref, text, value, zeroOrMore)
start.model = sims.Elements(attribute, choice, data, element_, empty, externalRef, grammar, group, interleave, list, mixed, notAllowed, oneOrMore, optional, parentRef, ref, text, value, zeroOrMore)
zeroOrMore.model = sims.Elements(choice, data, element_, empty, externalRef, grammar, group, interleave, list, mixed, notAllowed, oneOrMore, optional, parentRef, ref, text, value, zeroOrMore)

class xmlns(xsc.Namespace):
	xmlname = "rng"
	xmlurl = "http://relaxng.org/ns/structure/1.0"
xmlns.makemod(vars())
